using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using DerivativeFunctions;
using Experiments;
using LinearAlgebra;
using Steppers;

namespace NumericsOfDifferentialEquations
{
	/// <summary>
	/// Demonstrates an experiment in differential equations.
	/// </summary>
	public class MainForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Button buttonPause;
		private System.Windows.Forms.Button buttonExit;
		private System.Windows.Forms.Label labelTime;
		private System.ComponentModel.Container components = null;
		private System.Timers.Timer myTimer;

		private Experiment ex;

		static void Main() 
		{
			MainForm f = new MainForm();
			Application.Run(f);
		}

		public MainForm()
		{
			InitializeComponent();

			myTimer = new System.Timers.Timer();
			myTimer.Elapsed += new System.Timers.ElapsedEventHandler(onTimer);
			myTimer.Interval = 10;
			myTimer.Start();

			// Beispiel 1
//			ex = new CompareMethods(false, new HarmonicOscillator(1.0, 1.0), 0.3, new VectorN(1.23, 2.34),
//				new RungeKutta4());

			// Beispiel 2
//			ex = new CompareMethods(false, new HarmonicOscillator(1.0, 1.0), 0.3, new VectorN(1.23, 2.34),
//				new Euler(), new Midpoint(), new Heun(), new RungeKutta4());

			// Beispiel 2a
//			ex = new CompareMethods(false, new HarmonicOscillator(1.0, 1.0), 0.1, new VectorN(1.23, 2.34),
//				new Euler(), new Midpoint(), new Heun(), new RungeKutta4());

			// Beispiel 3
//			ex = new TestPrecision(new ExponentialGrowth(1.0), 1.0, 1.0, new VectorN(1.0), Math.E,
//				new Euler(), new Midpoint(), new Heun(), new RungeKutta4());

			// Beispiel 4
//			ex = new CompareMethods(false, new StrangeInstabilityEffect(), 0.15, new VectorN(0.0),
//				new Euler(), new Midpoint(), new Heun(), new RungeKutta4());

			// Beispiel 4a
//			ex = new CompareMethods(false, new StrangeInstabilityEffect(), 0.10, new VectorN(0.0),
//				new Euler(), new Midpoint(), new Heun(), new RungeKutta4());

			// Beispiel 5
//			ex = new CompareMethods(false, new StrangeInstabilityEffect(), 0.15, new VectorN(0.0),
//				new Euler(), new SemiImplicitEuler(), new Heun(), new RungeKutta4());

			// Beispiel 6
//			ex = new ShowStabilityRegion(new Euler(), new Midpoint(), new Heun(), new RungeKutta4());

			// Beispiel 7
//			ex = new CompareMethods(true, new SpringPendulum2D(1.0, 1.0, 10000.0, 9.81, 1.0), 0.003,
//				new VectorN(Math.Sqrt(0.5)+0.05, 0.0, -Math.Sqrt(0.5), 0.0),
//				new Euler(), new Midpoint(), new Heun(), new RungeKutta4());

			// Beispiel 7a
//			ex = new CompareMethods(true, new SpringPendulum2D(1.0, 1.0, 1000.0, 9.81, 1.0), 0.02,
//				new VectorN(Math.Sqrt(0.5), 0.0, -Math.Sqrt(0.5), 0.0),
//				new Euler(), new Midpoint(), new Heun(), new RungeKutta4());

			// Beispiel 7b
//			ex = new CompareMethods(true, new SpringPendulum2D(1.0, 1.0, 10000.0, 9.81, 1.0), 0.02,
//				new VectorN(Math.Sqrt(0.5), 0.0, -Math.Sqrt(0.5), 0.0),
//				new Euler(), new Midpoint(), new Heun(), new RungeKutta4());

			// Beispiel 8
//			ex = new CompareDistanceGrowth(new HarmonicOscillator(1.0, 2.0), 0.02, new VectorN(1.0, 2.0, 3.0, 4.0),
//				new VectorN(1.0 + 1e-5, 2.0, 3.0, 4.0), new RungeKutta4());

			// Beispiel 9
			ex = new CompareDistanceGrowth(new Rssler(), 0.1, new VectorN(1.0, 2.0, 0.0),
				new VectorN(1.0 + 1e-6, 2.0, 0.0), new RungeKutta4());		
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
			ex.Dispose();
		}

		#region Vom Windows Form-Designer generierter Code
		/// <summary>
		/// Erforderliche Methode fr die Designeruntersttzung. 
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		private void InitializeComponent()
		{
			this.buttonPause = new System.Windows.Forms.Button();
			this.buttonExit = new System.Windows.Forms.Button();
			this.labelTime = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// buttonPause
			// 
			this.buttonPause.ForeColor = System.Drawing.Color.DarkGreen;
			this.buttonPause.Location = new System.Drawing.Point(16, 32);
			this.buttonPause.Name = "buttonPause";
			this.buttonPause.TabIndex = 0;
			this.buttonPause.Text = "Pause/Cont.";
			this.buttonPause.Click += new System.EventHandler(this.buttonPause_Click);
			// 
			// buttonExit
			// 
			this.buttonExit.Location = new System.Drawing.Point(16, 64);
			this.buttonExit.Name = "buttonExit";
			this.buttonExit.TabIndex = 1;
			this.buttonExit.Text = "Exit";
			this.buttonExit.Click += new System.EventHandler(this.buttonExit_Click);
			// 
			// labelTime
			// 
			this.labelTime.Location = new System.Drawing.Point(16, 8);
			this.labelTime.Name = "labelTime";
			this.labelTime.Size = new System.Drawing.Size(104, 16);
			this.labelTime.TabIndex = 2;
			this.labelTime.Text = "---";
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(106, 94);
			this.Controls.Add(this.labelTime);
			this.Controls.Add(this.buttonExit);
			this.Controls.Add(this.buttonPause);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "MainForm";
			this.Opacity = 0.7;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "DiffEqn";
			this.TopMost = true;
			this.ResumeLayout(false);

		}
		#endregion

		private void buttonPause_Click(object sender, System.EventArgs e)
		{
			myTimer.Enabled = !myTimer.Enabled;
			if(myTimer.Enabled)
			{
				buttonPause.ForeColor = Color.DarkGreen;
			}
			else
			{
				buttonPause.ForeColor = Color.DarkRed;
			}
		}

		private void buttonExit_Click(object sender, System.EventArgs e)
		{
			Application.Exit();
		}

		private void onTimer(object sender, System.Timers.ElapsedEventArgs e)
		{
			string s = ex.Step();
			labelTime.Text = s;
		}
	}
}
